/*
  Copyright (C) 2013 Jolla Ltd.
  Contact: Thomas Perl <thomas.perl@jollamobile.com>
  All rights reserved.

  You may use this file under the terms of BSD license as follows:

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the Jolla Ltd nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS OR CONTRIBUTORS BE LIABLE FOR
  ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

import QtQuick 2.0
import Sailfish.Silica 1.0
import org.nemomobile.dbus 2.0

ApplicationWindow {
    initialPage: Component {
        Page {
            DBusInterface {
                id: dbusServer
                service: 'harbour.buatsap.Service'
                iface: 'harbour.buatsap.Server'
                path: '/'
            }

            DBusInterface {
                id: dbusChats
                service: 'harbour.buatsap.Service'
                iface: 'harbour.buatsap.Chats'
                path: '/chats'
                signalsEnabled: true

                function newChat(chatId, lastMessage){
                    console.log("chatId: "+chatId+" lastMessage: "+lastMessage)
                    messageModel.append({
                      id: chatId,
                      message: lastMessage
                    })
                }
            }

            SilicaFlickable {
                anchors.fill: parent
                contentHeight: column.height

                Column {
                    id: column
                    width: parent.width
                    height: childRect.height

                    PageHeader { title: "Qml DBus Client" }

                    TextField {
                        id: username
                        width: parent.width
                        label: "Username"
                        placeholderText: "Username"
                        focus: true
                        EnterKey.onClicked: {
                            password.focus = true;
                        }
                    }

                    TextField {
                        id: password
                        width: parent.width
                        inputMethodHints: Qt.ImhNoPredictiveText
                        echoMode: TextInput.Password
                        label: "Password"
                        placeholderText: "Enter password"
                        EnterKey.onClicked: {
                            dbusServer.typedCall('Login', [
                                                  { type: "s", value: username.text},
                                                  { type: "s", value: password.text}
                                               ], function(r){
                                console.log(r)
                                if(r)
                                    result.text = "Login succesfull!"
                                else
                                    result.text = "Bad login"
                            })
                        }
                    }

                    TextField {
                        id: result
                        width: parent.width
                        readOnly: true
                        label: ""
                        text: "Please login"
                    }

                    Repeater {
                        id: listView
                        width: parent.width
                        model: ListModel { id: messageModel }
                        height: childrenRect.height
                        delegate: BackgroundItem {
                            width: listView.width
                            Label {
                                text: "#"+ model.id +" "+model.message
                                color: highlighted ? Theme.highlightColor : Theme.primaryColor
                                anchors.verticalCenter: parent.verticalCenter
                                x: Theme.paddingLarge
                            }
                        }
                    }

                    Button {
                        text: "Test Signal"
                        onClicked: {
                            dbusChats.call("EmitNewChat", []);
                        }
                        anchors.horizontalCenter: parent.horizontalCenter
                    }
                }
            }
        }
    }
}
